@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 # *******************************************************
-# Copyright (c) VMware, Inc. 2020-2021. All Rights Reserved.
+# Copyright (c) VMware, Inc. 2020-2022. All Rights Reserved.
 # SPDX-License-Identifier: MIT
 # *******************************************************
 # *
@@ -38,6 +38,37 @@ class CreatableModelMixin(object):
     pass
 
 
+class SwaggerLoader(yaml.SafeLoader):
+    """YAML loader class for loading Swagger metafiles."""
+    _model_base_directory = os.path.dirname(__file__)
+
+
+def construct_include(loader, node):
+    """
+    Include the file referenced by the node.
+
+    Args:
+        loader (yaml.Loader): YAML loader object.
+        node (yaml.Node): Current node being loaded.
+
+    Returns:
+        Any: The data to be included in the YAML loader output.
+    """
+    filename = os.path.abspath(os.path.join(loader._model_base_directory, loader.construct_scalar(node)))
+    extension = os.path.splitext(filename)[1].lstrip('.')
+
+    with open(filename, 'rb') as f:
+        if extension in ('yaml', 'yml'):
+            return yaml.load(f, SwaggerLoader)
+        elif extension in ('json', ):
+            return json.load(f)
+        else:
+            return ''.join(f.readlines())
+
+
+yaml.add_constructor('!include', construct_include, SwaggerLoader)
+
+
 class CbMetaModel(type):
     """Meta-model for NewBaseModel and its subclasses."""
     model_base_directory = os.path.dirname(__file__)
@@ -55,14 +86,27 @@ class CbMetaModel(type):
         swagger_meta_file = clsdict.pop("swagger_meta_file", None)
         model_data = {}
         if swagger_meta_file:
-            model_data = yaml.safe_load(
-                open(os.path.join(mcs.model_base_directory, swagger_meta_file), 'rb').read())
+            model_data = yaml.load(
+                open(os.path.join(mcs.model_base_directory, swagger_meta_file), 'rb').read(), SwaggerLoader)
 
-        clsdict["__doc__"] = "Represents a %s object in the Carbon Black server.\n\n" % (name,)
+        # clsdict["__doc__"] = "Represents a %s object in the Carbon Black server.\n\n" % (name,)
+        # for field_name, field_info in iter(model_data.get("properties", {}).items()):
+        #    docstring = field_info.get("description", None)
+        #    if docstring:
+        #        clsdict["__doc__"] += ":ivar %s: %s\n" % (field_name, docstring)
+
+        class_docstr = clsdict.get('__doc__', None)
+        if not class_docstr:
+            class_docstr = f"Represents a {name} object in the Carbon Black Cloud."
+        need_header = True
         for field_name, field_info in iter(model_data.get("properties", {}).items()):
             docstring = field_info.get("description", None)
             if docstring:
-                clsdict["__doc__"] += ":ivar %s: %s\n" % (field_name, docstring)
+                if need_header:
+                    class_docstr += "\n\nParameters:"
+                    need_header = False
+                class_docstr += f"\n    {field_name}: {docstring}"
+        clsdict['__doc__'] = class_docstr
 
         foreign_keys = clsdict.pop("foreign_keys", {})
 
@@ -567,7 +611,9 @@ class NewBaseModel(object, metaclass=CbMetaModel):
 
         Notes:
             Should be overridden for any class wanting to use subobject reporting capabilities.
+
             If the attribute is a list of subobjects, this method should return that list.
+
             If the attribute is a single subobject, this method should return that subobject.
         """
         return None
@@ -634,7 +680,7 @@ class NewBaseModel(object, metaclass=CbMetaModel):
                 status = "(*)"
         subobject_value = self._subobject(name) if top_level else None
         spacing = name_field_len + (6 if top_level else 2)  # (status + space) + name + colon + space
-        if isinstance(value, list):
+        if isinstance(value, list) or isinstance(subobject_value, list):
             # this is a list - render the first three items, then [...] if we have more
             target = value if subobject_value is None else subobject_value
             list_header = f"[list:{len(target)} {'item' if len(target) == 1 else 'items'}]" \
@@ -804,9 +850,10 @@ class MutableBaseModel(NewBaseModel):
         if self._refresh():
             self._dirty_attributes = {}
 
-    def touch(self):
+    def touch(self, fulltouch=False):
         """Force this object to be considered as changed."""
-        self._dirty_attributes = {key: self._info.get(key, None) for key in self.__class__._valid_fields}
+        self._dirty_attributes = {key: self._info.get(key, None) for key in self.__class__._valid_fields
+                                  if key != self.__class__.primary_key or fulltouch}
 
     def is_dirty(self):
         """
@@ -827,7 +874,7 @@ class MutableBaseModel(NewBaseModel):
                 pass
             log.debug("Creating a new {0:s} object".format(self.__class__.__name__))
             http_method = self.__class__._new_object_http_method
-            ret = self._cb.api_json_request(http_method, self.urlobject,
+            ret = self._cb.api_json_request(http_method, self._build_api_request_uri(http_method=http_method),
                                             data=new_object_info)
         else:
             log.debug("Updating {0:s} with unique ID {1:s}".format(self.__class__.__name__, str(self._model_unique_id)))
@@ -1345,12 +1392,11 @@ class QueryBuilder(object):
     through the CBCloudAPI.select API.
 
     Examples:
-    >>> from cbc_sdk.base import QueryBuilder
-    >>> # build a query with chaining
-    >>> query = QueryBuilder().where(process_name="malicious.exe").and_(device_name="suspect")
-    >>> # start with an initial query, and chain another condition to it
-    >>> query = QueryBuilder(device_os="WINDOWS").or_(process_username="root")
-
+        >>> from cbc_sdk.base import QueryBuilder
+        >>> # build a query with chaining
+        >>> query = QueryBuilder().where(process_name="malicious.exe").and_(device_name="suspect")
+        >>> # start with an initial query, and chain another condition to it
+        >>> query = QueryBuilder(device_os="WINDOWS").or_(process_username="root")
     """
 
     def __init__(self, **kwargs):
@@ -1610,8 +1656,8 @@ class CriteriaBuilderSupportMixin:
             The query object with specified custom criteria.
 
         Example:
-            query = api.select(Event).add_criteria("event_type", ["filemod", "scriptload"])
-            query = api.select(Event).add_criteria("event_type", "filemod")
+            >>> query = api.select(Event).add_criteria("event_type", ["filemod", "scriptload"])
+            >>> query = api.select(Event).add_criteria("event_type", "filemod")
         """
         if not isinstance(newlist, list):
             if not isinstance(newlist, str):
@@ -1633,9 +1679,10 @@ class CriteriaBuilderSupportMixin:
             The query object with specified custom criteria.
 
         Example:
-            query = api.select(Alert).update_criteria("my.criteria.key", ["criteria_value"])
+            >>> query = api.select(Alert).update_criteria("my.criteria.key", ["criteria_value"])
 
-        Note: Use this method if there is no implemented method for your desired criteria.
+        Note:
+            Use this method if there is no implemented method for your desired criteria.
         """
         if not isinstance(newlist, list):
             if not isinstance(newlist, str):
@@ -1697,17 +1744,17 @@ class AsyncQueryMixin:
 
 
 class Query(PaginatedQuery, QueryBuilderSupportMixin, IterableQueryMixin, AsyncQueryMixin, CriteriaBuilderSupportMixin):
-    """Represents a prepared query to the Cb Enterprise EDR backend.
+    """Represents a prepared query to the Carbon Black Cloud.
 
-    This object is returned as part of a `CbEnterpriseEDRAPI.select`
-    operation on models requested from the Cb Enterprise EDR backend. You should not have to create this class yourself.
+    This object is returned as part of a `CBCCloudAPI.select`
+    operation on models requested from the Carbon Black Cloud backend.
+    You should not have to create this class yourself.
 
     The query is not executed on the server until it's accessed, either as an iterator (where it will generate values
     on demand as they're requested) or as a list (where it will retrieve the entire result set and save to a list).
     You can also call the Python built-in ``len()`` on this object to retrieve the total number of items matching
     the query.
 
-    Examples::
 
     >>> from cbc_sdk import CBCloudAPI
     >>> from cbc_sdk.enterprise_edr import Report
@@ -1756,8 +1803,8 @@ class Query(PaginatedQuery, QueryBuilderSupportMixin, IterableQueryMixin, AsyncQ
             The ResultQuery with specified custom exclusion.
 
         Example:
-            query = api.select(Event).add_exclusions("netconn_domain", ["www.google.com"])
-            query = api.select(Event).add_exclusions("netconn_domain", "www.google.com")
+            >>> query = api.select(Event).add_exclusions("netconn_domain", ["www.google.com"])
+            >>> query = api.select(Event).add_exclusions("netconn_domain", "www.google.com")
         """
         if not isinstance(newlist, list):
             if not isinstance(newlist, str):
@@ -1835,9 +1882,10 @@ class Query(PaginatedQuery, QueryBuilderSupportMixin, IterableQueryMixin, AsyncQ
             - `window` will take precendent over `start` and `end` if provided.
 
         Examples:
-            query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z")
-            second_query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z", end="2020-10-30T20:34:07Z")
-            third_query = api.select(Event).set_time_range(window='-3d')
+            >>> query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z")
+            >>> second_query = api.select(Event).
+            ...     set_time_range(start="2020-10-20T20:34:07Z", end="2020-10-30T20:34:07Z")
+            >>> third_query = api.select(Event).set_time_range(window='-3d')
         """
         if start:
             if not isinstance(start, str):
@@ -1885,8 +1933,7 @@ class Query(PaginatedQuery, QueryBuilderSupportMixin, IterableQueryMixin, AsyncQ
             Query: The query with sorting parameters.
 
         Example:
-
-        >>> cb.select(Process).where(process_name="cmd.exe").sort_by("device_timestamp")
+            >>> cb.select(Process).where(process_name="cmd.exe").sort_by("device_timestamp")
         """
         found = False
 
@@ -2007,8 +2054,8 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
             The ResultQuery with specified custom exclusion.
 
         Example:
-            query = api.select(Event).add_exclusions("netconn_domain", ["www.google.com"])
-            query = api.select(Event).add_exclusions("netconn_domain", "www.google.com")
+            >>> query = api.select(Event).add_exclusions("netconn_domain", ["www.google.com"])
+            >>> query = api.select(Event).add_exclusions("netconn_domain", "www.google.com")
         """
         if not isinstance(newlist, list):
             if not isinstance(newlist, str):
@@ -2038,11 +2085,10 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
 
         Returns:
             Query (AsyncQuery): The Query object with new milliseconds
-                parameter.
+            parameter.
 
         Example:
-
-        >>> cb.select(ProcessFacet).where(process_name="foo.exe").timeout(5000)
+            >>> cb.select(ProcessFacet).where(process_name="foo.exe").timeout(5000)
         """
         self._timeout = msecs
         return self
@@ -2059,7 +2105,7 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
             Query (AsyncQuery): The Query object with new limit parameter.
 
         Example:
-        >>> cb.select(ProcessFacet).where(process_name="foo.exe").limit(50)
+            >>> cb.select(ProcessFacet).where(process_name="foo.exe").limit(50)
         """
         self._limit = limit
         return self
@@ -2074,7 +2120,7 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
             Query (AsyncQuery): The Query object with the new rows parameter.
 
         Example:
-        >>> cb.select(ProcessFacet).set_rows(50)
+            >>> cb.select(ProcessFacet).set_rows(50)
         """
         self._facet_rows = rows
         return self
@@ -2089,7 +2135,7 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
             Query (AsyncQuery): The Query object that will receive the specified field(s).
 
         Example:
-        >>> cb.select(ProcessFacet).add_facet_field(["process_name", "process_username"])
+            >>> cb.select(ProcessFacet).add_facet_field(["process_name", "process_username"])
         """
         if isinstance(field, str):
             self._facet_fields.append(field)
@@ -2124,7 +2170,8 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
             raise ApiError("`bucket_size` should be either int or ISO8601 timestamp string")
 
     def add_range(self, range):
-        """Sets the facet ranges to be received by this query.
+        """
+        Sets the facet ranges to be received by this query.
 
         Arguments:
             range (dict or [dict]): Range(s) to be received.
@@ -2132,20 +2179,27 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
         Returns:
             Query (AsyncQuery): The Query object that will receive the specified range(s).
 
-        Note: The range parameter must be in this dictionary format:
+        Note:
+            The range parameter must be in this dictionary format:
+
             {
+
                 "bucket_size": "<object>",
+
                 "start": "<object>",
+
                 "end": "<object>",
+
                 "field": "<string>"
+
             },
+
             where "bucket_size", "start", and "end" can be numbers or ISO 8601 timestamps.
 
         Examples:
-        >>> cb.select(ProcessFacet).add_range({"bucket_size": 5, "start": 0, "end": 10, "field": "netconn_count"})
-        >>> cb.select(ProcessFacet).add_range({"bucket_size": "+1DAY", "start": "2020-11-01T00:00:00Z",
-                                               "end": "2020-11-12T00:00:00Z", "field": "backend_timestamp"})
-
+            >>> cb.select(ProcessFacet).add_range({"bucket_size": 5, "start": 0, "end": 10, "field": "netconn_count"})
+            >>> cb.select(ProcessFacet).add_range({"bucket_size": "+1DAY", "start": "2020-11-01T00:00:00Z",
+            ... "end": "2020-11-12T00:00:00Z", "field": "backend_timestamp"})
         """
         if isinstance(range, dict):
             self._check_range(range)
@@ -2164,15 +2218,16 @@ class FacetQuery(BaseQuery, AsyncQueryMixin, QueryBuilderSupportMixin, CriteriaB
             start (str in ISO 8601 timestamp): When to start the result search.
             end (str in ISO 8601 timestamp): When to end the result search.
             window (str): Time window to execute the result search, ending on the current time.
-                Should be in the form "-2w", where y=year, w=week, d=day, h=hour, m=minute, s=second.
+            Should be in the form "-2w", where y=year, w=week, d=day, h=hour, m=minute, s=second.
 
         Note:
             - `window` will take precendent over `start` and `end` if provided.
 
         Examples:
-            query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z")
-            second_query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z", end="2020-10-30T20:34:07Z")
-            third_query = api.select(Event).set_time_range(window='-3d')
+            >>> query = api.select(Event).set_time_range(start="2020-10-20T20:34:07Z")
+            >>> second_query = api.select(Event).
+            ...     set_time_range(start="2020-10-20T20:34:07Z", end="2020-10-30T20:34:07Z")
+            >>> third_query = api.select(Event).set_time_range(window='-3d')
         """
         if start:
             if not isinstance(start, str):
