@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 # *******************************************************
-# Copyright (c) VMware, Inc. 2020-2021. All Rights Reserved.
+# Copyright (c) VMware, Inc. 2020-2022. All Rights Reserved.
 # SPDX-License-Identifier: MIT
 # *******************************************************
 # *
@@ -15,10 +15,13 @@
 
 from __future__ import absolute_import
 
+import pkgutil
+import cbc_sdk
 import requests
 import sys
 from requests.adapters import HTTPAdapter, DEFAULT_POOLBLOCK, DEFAULT_RETRIES, DEFAULT_POOLSIZE, DEFAULT_POOL_TIMEOUT
 
+
 try:
     from requests.packages.urllib3.util.ssl_ import create_urllib3_context
     REQUESTS_HAS_URLLIB_SSL_CONTEXT = True
@@ -44,11 +47,11 @@ import urllib
 from .credentials import Credentials
 from .credential_providers.default import default_credential_provider
 from .errors import ClientError, QuerySyntaxError, ServerError, TimeoutError, ApiError, ObjectNotFoundError, \
-    UnauthorizedError, ConnectionError
+    UnauthorizedError, ConnectionError, ModelNotFound
 from . import __version__
 
 from .cache.lru import lru_cache_function
-from .base import CreatableModelMixin
+from .base import CreatableModelMixin, NewBaseModel
 from .utils import convert_query_params
 
 log = logging.getLogger(__name__)
@@ -180,7 +183,8 @@ class Connection(object):
         if not credentials.url or not credentials.url.startswith("https://"):
             raise ConnectionError("Server URL must be a URL: eg. https://localhost")
 
-        if not credentials.token:
+        self.token = credentials.get_token()
+        if not self.token:
             raise ConnectionError("No API token provided")
 
         self.server = credentials.url.rstrip("/")
@@ -201,8 +205,11 @@ class Connection(object):
         if integration_name:
             user_agent = f"{integration_name} {user_agent}"
 
-        self.token = credentials.token
-        self.token_header = {'X-Auth-Token': self.token, 'User-Agent': user_agent}
+        if credentials.get_token_type() == "BEARER":
+            self.token_header = {'Authorization': f"Bearer {self.token}", 'User-Agent': user_agent}
+        else:
+            self.token_header = {'X-Auth-Token': self.token, 'User-Agent': user_agent}
+
         if proxy_session:
             self.session = proxy_session
             credentials.use_custom_proxy_session = True
@@ -228,6 +235,22 @@ class Connection(object):
 
         self.session.mount(self.server, tls_adapter)
 
+        def refresh_token(r, *args, **kwargs):
+            """Hook to refresh expired OAuth tokens"""
+            if r.status_code == 401 and credentials.get_token_type() == "BEARER":
+                log.debug("Fetching new bearer token as the previous token expired")
+                self.token = credentials.get_token()
+
+                # Update Headers
+                self.token_header.update({"Authorization": f"Bearer {self.token}"})
+                self.session.headers.update({"Authorization": f"Bearer {self.token}"})
+                r.request.headers["Authorization"] = self.session.headers["Authorization"]
+
+                # Resend Request
+                return self.session.send(r.request, **kwargs)
+
+        self.session.hooks["response"].append(refresh_token)
+
         self.proxies = {}
         if credentials.use_custom_proxy_session:
             # get the custom session proxies
@@ -281,7 +304,6 @@ class Connection(object):
             headers = self.token_header
 
         uri = self.server + url
-
         try:
             raw_data = kwargs.get("data", None)
             if raw_data:
@@ -380,8 +402,13 @@ class BaseAPI(object):
         integration_name = kwargs.pop("integration_name", None)
         self.credential_provider = kwargs.pop("credential_provider", None)
 
-        url, token = kwargs.get("url", None), kwargs.get("token", None)
-        if url and token:
+        url = kwargs.get("url", None)
+        token = kwargs.get("token", None)
+        csp_api_token = kwargs.get("csp_api_token", None)
+        csp_oauth_app_id = kwargs.get("csp_oauth_app_id", None)
+        csp_oauth_app_secret = kwargs.get("csp_oauth_app_secret", None)
+
+        if url and (token or csp_api_token or (csp_oauth_app_id and csp_oauth_app_secret)):
             self.credentials = Credentials(kwargs)
             self.credentials.integration = integration_name
             self.credential_profile_name = None
@@ -665,7 +692,7 @@ class BaseAPI(object):
         Prepare a query against the Carbon Black data store.
 
         Args:
-            cls (class): The Model class (for example, Computer, Process, Binary, FileInstance) to query
+            cls (class | str): The Model class (for example, Computer, Process, Binary, FileInstance) to query
             unique_id (optional): The unique id of the object to retrieve, to retrieve a single object by ID
             *args:
             **kwargs:
@@ -673,6 +700,8 @@ class BaseAPI(object):
         Returns:
             object: An instance of the Model class if a unique_id is provided, otherwise a Query object
         """
+        if isinstance(cls, str):
+            cls = select_class_instance(cls)
         if unique_id is not None:
             return select_instance(self, cls, unique_id, *args, **kwargs)
         else:
@@ -733,3 +762,34 @@ def select_instance(api, cls, unique_id, *args, **kwargs):
         object: New object instance.
     """
     return cls(api, unique_id, *args, **kwargs)
+
+
+def select_class_instance(cls: str):
+    """
+    Selecting the appropriate class based on the passed string.
+
+    Args:
+        cls: The class name represented in a string.
+
+    Returns:
+        Object[]:
+    """
+    # Walk through all the packages contained in the `cbc_sdk`, ensures the loading
+    # of all the needed packages.
+    _ = [i for i in pkgutil.walk_packages(cbc_sdk.__path__, f"{cbc_sdk.__name__}.")]
+
+    subclasses = set()
+    base_subclasses = NewBaseModel.__subclasses__().copy()
+    while base_subclasses:
+        parent = base_subclasses.pop()
+        subclasses.add(parent)
+        for child in parent.__subclasses__():
+            if child not in subclasses:
+                subclasses.add(child)
+                base_subclasses.append(child)
+
+    # https://www.python.org/dev/peps/pep-3155/#rationale
+    lookup_dict = {klass.__qualname__: klass for klass in subclasses}
+    if cls in lookup_dict.keys():
+        return lookup_dict[cls]
+    raise ModelNotFound()
