@@ -2,6 +2,8 @@
 
 from __future__ import absolute_import
 
-from cbc_sdk.endpoint_standard.base import Event, Policy, EnrichedEvent, EnrichedEventFacet 
+from cbc_sdk.endpoint_standard.base import Event, EnrichedEvent, EnrichedEventFacet
 from cbc_sdk.endpoint_standard.usb_device_control import USBDeviceApproval, USBDeviceBlock, USBDevice
 from cbc_sdk.endpoint_standard.recommendation import Recommendation
+
+from cbc_sdk.platform.policies import Policy
