@@ -3,12 +3,14 @@ from __future__ import absolute_import
 from cbc_sdk.platform.base import PlatformModel
 
 from cbc_sdk.platform.alerts import (BaseAlert, WatchlistAlert, CBAnalyticsAlert, DeviceControlAlert,
-                                     Workflow, WorkflowStatus)
+                                     ContainerRuntimeAlert, Workflow, WorkflowStatus)
 
-from cbc_sdk.platform.devices import Device, DeviceSearchQuery
+from cbc_sdk.platform.devices import Device, DeviceFacet, DeviceSearchQuery
 
 from cbc_sdk.platform.events import Event, EventFacet
 
+from cbc_sdk.platform.policies import Policy, PolicyRule
+
 from cbc_sdk.platform.processes import (Process, ProcessFacet,
                                         AsyncProcessQuery, SummaryQuery)
 
