@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 # *******************************************************
-# Copyright (c) VMware, Inc. 2020-2021. All Rights Reserved.
+# Copyright (c) VMware, Inc. 2020-2022. All Rights Reserved.
 # SPDX-License-Identifier: MIT
 # *******************************************************
 # *
@@ -13,11 +13,12 @@
 
 """Model and Query Classes for Platform Devices"""
 
-from cbc_sdk.errors import ApiError, ServerError
+from cbc_sdk.errors import ApiError, ServerError, NonQueryableModel
 from cbc_sdk.platform import PlatformModel
 from cbc_sdk.platform.vulnerability_assessment import Vulnerability, VulnerabilityQuery
-from cbc_sdk.base import (BaseQuery, QueryBuilder, QueryBuilderSupportMixin, CriteriaBuilderSupportMixin,
-                          IterableQueryMixin, AsyncQueryMixin)
+from cbc_sdk.base import (UnrefreshableModel, BaseQuery, QueryBuilder, QueryBuilderSupportMixin,
+                          CriteriaBuilderSupportMixin, IterableQueryMixin, AsyncQueryMixin)
+from cbc_sdk.workload import NSXRemediationJob
 
 import time
 
@@ -72,6 +73,9 @@ class Device(PlatformModel):
         """
         Rereads the device data from the server.
 
+        Required Permissions:
+            device (READ)
+
         Returns:
             bool: True if refresh was successful, False if not.
         """
@@ -85,6 +89,9 @@ class Device(PlatformModel):
         """
         Retrieve a Live Response session object for this Device.
 
+        Required Permissions:
+            org.liveresponse.session (CREATE)
+
         Returns:
             LiveResponseSession: Live Response session for the Device.
 
@@ -97,6 +104,9 @@ class Device(PlatformModel):
         """
         Set the background scan option for this device.
 
+        Required Permissions:
+            device.bg-scan (EXECUTE)
+
         Args:
             flag (bool): True to turn background scan on, False to turn it off.
 
@@ -109,6 +119,9 @@ class Device(PlatformModel):
         """
         Set the bypass option for this device.
 
+        Required Permissions:
+            device.bypass (EXECUTE)
+
         Args:
             flag (bool): True to enable bypass, False to disable it.
 
@@ -121,6 +134,9 @@ class Device(PlatformModel):
         """
         Delete this sensor device.
 
+        Required Permissions:
+            device.deregistered (DELETE)
+
         Returns:
             str: The JSON output from the request.
         """
@@ -130,6 +146,9 @@ class Device(PlatformModel):
         """
         Uninstall this sensor device.
 
+        Required Permissions:
+            device.uninstall (EXECUTE)
+
         Returns:
             str: The JSON output from the request.
         """
@@ -139,6 +158,9 @@ class Device(PlatformModel):
         """
         Set the quarantine option for this device.
 
+        Required Permissions:
+            device.quarantine (EXECUTE)
+
         Args:
             flag (bool): True to enable quarantine, False to disable it.
 
@@ -151,6 +173,9 @@ class Device(PlatformModel):
         """
         Set the current policy for this device.
 
+        Required Permissions:
+            device.policy (UPDATE)
+
         Args:
             policy_id (int): ID of the policy to set for the devices.
 
@@ -163,6 +188,9 @@ class Device(PlatformModel):
         """
         Update the sensor version for this device.
 
+        Required Permissions:
+            org.kits (EXECUTE)
+
         Args:
             sensor_version (dict): New version properties for the sensor.
 
@@ -172,7 +200,12 @@ class Device(PlatformModel):
         return self._cb.device_update_sensor_version([self._model_unique_id], sensor_version)
 
     def vulnerability_refresh(self):
-        """Perform an action on a specific device. Only REFRESH is supported."""
+        """
+        Perform an action on a specific device. Only REFRESH is supported.
+
+        Required Permissions:
+            vulnerabilityAssessment.data (EXECUTE)
+        """
         request = {"action_type": 'REFRESH'}
         url = "/vulnerability/assessment/api/v1/orgs/{}".format(self._cb.credentials.org_key)
 
@@ -190,6 +223,9 @@ class Device(PlatformModel):
         """
         Get the vulnerabilities associated with this device
 
+        Required Permissions:
+            vulnerabilityAssessment.data (READ)
+
         Args:
             category (string): (optional) vulnerabilty category (OS, APP)
 
@@ -219,6 +255,144 @@ class Device(PlatformModel):
         """
         return VulnerabilityQuery(Vulnerability, self._cb, self)
 
+    @property
+    def nsx_available(self):
+        """
+        Returns whether NSX actions are available on this device.
+
+        Returns:
+            bool: True if NSX actions are available, False if not.
+        """
+        return self._info['deployment_type'] == 'WORKLOAD' and self._info['nsx_enabled']
+
+    def nsx_remediation(self, tag, set_tag=True):
+        """
+        Start an NSX Remediation job on this device to change the tag.
+
+        Required Permissions:
+            appliances.nsx.remediation(EXECUTE)
+
+        Args:
+            tag (str): The NSX tag to apply to this device. Valid values are "CB-NSX-Quarantine",
+                       "CB-NSX-Isolate", and "CB-NSX-Custom".
+            set_tag (bool): True to toggle the specified tag on, False to toggle it off. Default True.
+
+        Returns:
+            NSXRemediationJob: The object representing all running jobs.  None if the operation is a no-op.
+        """
+        if not self.nsx_available:
+            raise ApiError("NSX actions are not available on this device")
+        current = self._info['nsx_distributed_firewall_policy']
+        if current is None:
+            if not set_tag:
+                return None  # clearing tag is a no-op if no tag is set
+        elif current == tag:
+            if set_tag:
+                return None  # setting tag is a no-op if already set
+        else:
+            if set_tag:
+                raise ApiError(f"NSX tag already set to {current}, cannot set another tag without clearing it")
+            return None  # clearing tag is a no-op in this case
+        return NSXRemediationJob.start_request(self._cb, self.id, tag, set_tag)
+
+
+class DeviceFacet(UnrefreshableModel):
+    """Represents a device field in a facet search."""
+    urlobject = "/appservices/v6/orgs/{0}/devices/_facet"
+    primary_key = "id"
+    swagger_meta_file = "platform/models/device_facet.yaml"
+
+    def __init__(self, cb, model_unique_id, initial_data=None):
+        """
+        Initialize the DeviceFacet object.
+
+        Args:
+            cb (BaseAPI): Reference to API object used to communicate with the server.
+            model_unique_id (str): Not used.
+            initial_data (dict): Initial data used to populate the facet.
+        """
+        super(DeviceFacet, self).__init__(cb, model_unique_id, initial_data, force_init=False, full_doc=True)
+        self._values = [DeviceFacet.DeviceFacetValue(cb, self, item['id'], item) for item in initial_data['values']]
+
+    class DeviceFacetValue(UnrefreshableModel):
+        """Represents a value of a particular field."""
+        def __init__(self, cb, outer, model_unique_id, initial_data):
+            """
+            Initialize the DeviceFacetValue object.
+
+            Args:
+                cb (BaseAPI): Reference to API object used to communicate with the server.
+                outer (DeviceFacet): Reference to outer facet object.
+                model_unique_id (str): Value ID.
+                initial_data (dict): Initial data used to populate the facet value.
+            """
+            super(DeviceFacet.DeviceFacetValue, self).__init__(cb, model_unique_id, initial_data, force_init=False,
+                                                               full_doc=True)
+            self._outer = outer
+
+        def query_devices(self):
+            """
+            Set up a device query to find all devices that match this facet value.
+
+            Example:
+                >>> facets = api.select(Device).where('').facets(['policy_id'])
+                >>> for value in facets[0].values_:
+                ...     print(f"Policy ID = {value.id}:")
+                ...     for dev in value.query_devices():
+                ...         print(f"    {dev.name} ({dev.last_external_ip_address})")
+
+            Returns:
+                DeviceQuery: A new DeviceQuery set with the criteria, which may have additional criteria added to it.
+            """
+            query = self._cb.select(Device)
+            if self._outer.field == 'policy_id':
+                query.set_policy_ids([int(self.id)])
+            elif self._outer.field == 'status':
+                query.set_status([self.id])
+            elif self._outer.field == 'os':
+                query.set_os([self.id.upper()])
+            elif self._outer.field == 'ad_group_id':
+                query.set_ad_group_ids([int(self.id)])
+            return query
+
+    @classmethod
+    def _query_implementation(cls, cb, **kwargs):
+        """
+        Returns the appropriate query object for the Device type.
+
+        Args:
+            cb (BaseAPI): Reference to API object used to communicate with the server.
+            **kwargs (dict): Not used, retained for compatibility.
+
+        Returns:
+            DeviceFacetQuery: The query object for this alert type.
+        """
+        raise NonQueryableModel("use facets() on DeviceQuery to get DeviceFacet")
+
+    def _subobject(self, name):
+        """
+        Returns the "subobject value" of the given attribute.
+
+        Args:
+            name (str): Name of the subobject value to be returned.
+
+        Returns:
+            Any: Subobject value for the attribute, or None if there is none.
+        """
+        if name == 'values':
+            return self._values
+        return super(DeviceFacet, self)._subobject(name)
+
+    @property
+    def values_(self):
+        """
+        Return the list of facet values for this facet.
+
+        Returns:
+            list[DeviceFacetValue]: The list of values for this facet.
+        """
+        return self._values
+
 
 ############################################
 # Device Queries
@@ -234,6 +408,7 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
     VALID_PRIORITIES = ["LOW", "MEDIUM", "HIGH", "MISSION_CRITICAL"]
     VALID_DIRECTIONS = ["ASC", "DESC"]
     VALID_DEPLOYMENT_TYPES = ["ENDPOINT", "WORKLOAD"]
+    VALID_FACET_FIELDS = ["policy_id", "status", "os", "ad_group_id"]
 
     def __init__(self, doc_class, cb):
         """
@@ -533,6 +708,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Returns the number of results from the run of this query.
 
+        Required Permissions:
+            device (READ)
+
         Returns:
             int: The number of results from the run of this query.
         """
@@ -555,6 +733,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
 
         Device v6 API uses base 1 instead of 0.
 
+        Required Permissions:
+            device (READ)
+
         Args:
             from_row (int): The row to start the query at (default 1).
             max_rows (int): The maximum number of rows to be returned (default -1, meaning "all").
@@ -593,6 +774,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Executed in the background to run an asynchronous query. Must be implemented in any inheriting classes.
 
+        Required Permissions:
+            device (READ)
+
         Args:
             context (object): The context returned by _init_async_query. May be None.
 
@@ -616,6 +800,45 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
             output += [self._doc_class(self._cb, item["id"], item) for item in results]
         return output
 
+    def facets(self, fieldlist, max_rows=0):
+        """
+        Return information about the facets for all known evices, using the defined criteria.
+
+        Example:
+            >>> query = api.select(Device).where('')
+            >>> facets = query.facets(['policy_id', 'status', 'os', 'ad_group_id'])
+            >>> for f in facets:
+            ...     print(f"Field {f.field} - {len(f.values_)} distinct values")
+
+        Required Permissions:
+            device (READ)
+
+        Args:
+            fieldlist (list[str]): List of facet field names. Valid names are "policy_id", "status", "os",
+                                   and "ad_group_id".
+            max_rows (int): The maximum number of rows to return. 0 means return all rows.
+
+        Returns:
+            list[DeviceFacet]: A list of facet information.
+        """
+        if not fieldlist:
+            raise ApiError("At least one term field must be specified")
+        if not all((field in DeviceSearchQuery.VALID_FACET_FIELDS) for field in fieldlist):
+            raise ApiError("One or more invalid term field names")
+        request = self._build_request(-1, -1)
+        if 'rows' in request:
+            del request['rows']
+        if 'sort' in request:
+            del request['sort']
+        terms = {'fields': fieldlist}
+        if max_rows > 0:
+            terms['rows'] = max_rows
+        request['terms'] = terms
+        url = self._build_url("/_facet")
+        resp = self._cb.post_object(url, body=request)
+        result = resp.json()
+        return [DeviceFacet(self._cb, None, item) for item in result['results']]
+
     def download(self):
         """
         Uses the query parameters that have been set to download all device listings in CSV format.
@@ -623,6 +846,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         Example:
             >>> cb.select(Device).set_status(["ALL"]).download()
 
+        Required Permissions:
+            device (READ)
+
         Returns:
             str: The CSV raw data as returned from the server.
 
@@ -655,6 +881,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Perform a bulk action on all devices matching the current search criteria.
 
+        Required Permissions:
+            Dependent on the action_type.
+
         Args:
             action_type (str): The action type to be performed.
             options (dict): Any options for the bulk device action.
@@ -671,6 +900,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Set the background scan option for the specified devices.
 
+        Required Permissions:
+            device.bg-scan (EXECUTE)
+
         Args:
             scan (bool): True to turn background scan on, False to turn it off.
 
@@ -683,6 +915,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Set the bypass option for the specified devices.
 
+        Required Permissions:
+            device.bypass (EXECUTE)
+
         Args:
             enable (bool): True to enable bypass, False to disable it.
 
@@ -695,6 +930,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Delete the specified sensor devices.
 
+        Required Permissions:
+            device.deregistered (DELETE)
+
         Returns:
             str: The JSON output from the request.
         """
@@ -704,6 +942,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Uninstall the specified sensor devices.
 
+        Required Permissions:
+            device.uninstall (EXECUTE)
+
         Returns:
             str: The JSON output from the request.
         """
@@ -713,6 +954,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Set the quarantine option for the specified devices.
 
+        Required Permissions:
+            device.quarantine (EXECUTE)
+
         Args:
             enable (bool): True to enable quarantine, False to disable it.
 
@@ -725,6 +969,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Set the current policy for the specified devices.
 
+        Required Permissions:
+            device.policy (UPDATE)
+
         Args:
             policy_id (int): ID of the policy to set for the devices.
 
@@ -737,6 +984,9 @@ class DeviceSearchQuery(BaseQuery, QueryBuilderSupportMixin, CriteriaBuilderSupp
         """
         Update the sensor version for the specified devices.
 
+        Required Permissions:
+            org.kits (EXECUTE)
+
         Args:
             sensor_version (dict): New version properties for the sensor.
 
