@@ -1,7 +1,7 @@
 #!/usr/bin/env python3
 
 # *******************************************************
-# Copyright (c) VMware, Inc. 2020-2021. All Rights Reserved.
+# Copyright (c) VMware, Inc. 2020-2022. All Rights Reserved.
 # SPDX-License-Identifier: MIT
 # *******************************************************
 # *
@@ -44,9 +44,10 @@ class CBCloudAPI(BaseAPI):
         Keyword Args:
             profile (str): Use the credentials in the named profile when connecting to the Carbon Black server.
                 Uses the profile named 'default' when not specified.
+            threat_pool_count (int): The number of threads to create for asynchronous queries. Defaults to 3.
         """
         super(CBCloudAPI, self).__init__(*args, **kwargs)
-        self._thread_pool_count = kwargs.pop('thread_pool_count', 1)
+        self._thread_pool_count = kwargs.pop('thread_pool_count', 3)
         self._lr_scheduler = None
         self._async_executor = None
 
@@ -114,7 +115,7 @@ class CBCloudAPI(BaseAPI):
             sql (str): The SQL for the query.
 
         Returns:
-            Query: The query object.
+            cbc_sdk.base.Query: The query object.
         """
         return self.select(Run).where(sql=sql)
 
@@ -126,7 +127,7 @@ class CBCloudAPI(BaseAPI):
             query (str): The SQL for the query.
 
         Returns:
-            Query: The query object.
+            cbc_sdk.base.Query: The query object.
         """
         return self.select(RunHistory).where(query)
 
@@ -418,7 +419,7 @@ class CBCloudAPI(BaseAPI):
             object: An instance of `cls`.
 
         Examples:
-        >>> feed = cb.create(Feed, feed_data)
+            >>> feed = cb.create(Feed, feed_data)
         """
         return cls(self, initial_data=data)
 
@@ -433,7 +434,7 @@ class CBCloudAPI(BaseAPI):
             bool: True if the query is valid, False if not.
 
         Examples:
-        >>> cb.validate_process_query("process_name:chrome.exe") # True
+            >>> cb.validate_process_query("process_name:chrome.exe") # True
         """
         args = {"q": query}
         url = "/api/investigate/v1/orgs/{}/processes/search_validation".format(
@@ -481,8 +482,8 @@ class CBCloudAPI(BaseAPI):
         """Returns a dictionary containing API limiting information.
 
         Examples:
-        >>> cb.process_limits()
-        {u'status_code': 200, u'time_bounds': {u'upper': 1545335070095, u'lower': 1542779216139}}
+            >>> cb.process_limits()
+            {u'status_code': 200, u'time_bounds': {u'upper': 1545335070095, u'lower': 1542779216139}}
         """
         url = "/api/investigate/v1/orgs/{}/processes/limits".format(
             self.credentials.org_key
